//--------------------------------------------------
// Dialog Control Class
//--------------------------------------------------
// $Revision:   1.13  $
// $Date:   24 Apr 1994 19:25:02  $
//--------------------------------------------------
#ifndef __OP_PANEL_H
#define __OP_PANEL_H
#ifndef __OPUS_H
#include "opus.h"
#endif
#ifndef __OP_GRAPH_H
#include "op_graph.h"
#endif
#ifndef __OP_MOUSE_H
#include "op_mouse.h"
#endif
#ifndef __OP_KEY_H
#include "op_key.h"
#endif
#ifndef __TIME_H
#include <time.h>
#endif

// Control types
#define ARROW             0
#define PUSH              1
#define TOGGLE            2
#define RADIO             3
#define PROMPT            4

// Event constants
#define NO_EVENT         -1
#define EXIT_EVENT       -2
#define NEXT_CONTROL     -3
#define PREV_CONTROL     -4
#define NO_GROUP         -1
#define NO_LABEL          0
#define LEFT_ARROW        1
#define RIGHT_ARROW       2
#define UP_ARROW          3
#define DOWN_ARROW        4

// Button constants
#define BTN_UP            2
#define BTN_DOWN          3
#define BTN_INACTIVE      4
#define NORMAL_BTN        0
#define SNAP_UP_BTN       1
#define HOLD_DOWN_BTN     2

// Led constants
#define LED_LONG          0
#define LED_DOUBLE        1
#define NO_BUTTONS        0 
#define LR_BTNS           1
#define UD_BTNS           2
#define LED_DISPLAY       0
#define LED_PRINT         1

// Radio buttons
#define RADIO_GROUPS    100
#define HORZ              0
#define VERT              1
#define DRAW_ALL          0
#define DRAW_BUTTONS      1
#define DRAW_LIT          2

// Prompts
#define DEFAULT_LENGTH   60
#define NULL_CHAR       '\0'
#define CANCEL            0
#define COMPLETED         1
#define EDITING           2
#define BLINK_CURSOR   0x01
#define SHOW_INPUT     0x02
#define SHOW_BORDER    0x04
#define ERASE_CURSOR   0x08
#define BEFORE_INPUT   0x10
#define DURING_INPUT   0x20
#define AFTER_INPUT    0x40
#define NO_PROMPT        -1
#define NEW_KEY        1000

// Built in button bitmaps
extern char l_up[];
extern char l_dn[];
extern char l_in[];
extern char r_up[];
extern char r_dn[];
extern char r_in[];
extern char u_up[];
extern char u_dn[];
extern char u_in[];
extern char d_up[];
extern char d_dn[];
extern char d_in[];

//----------------------------------------------------------------------
// General control item class
//----------------------------------------------------------------------
class Item
{
 private:
  int active;                 // Flags that control is currently active
  int paged;                  // Item is drawn twice: once for each page
  int split;                  // Item is drawn on a split screen

 public:
  Graphics *gr;               // Pointer to active Display class
  Graphics_font *grf;         // Pointer to graphics font class
  int font, font_size;        // Font description
  int x1, y1, x2, y2;         // Item's bounding rectangle
  int host_port;              // Viewer port in which to draw item

  Item(void);
  ~Item(void) {}
  void place(int xa, int ya, int xb, int yb)  // Define bounding rectangle
   { x1 = xa; y1 = ya; x2 = xb; y2 = yb; }
  void special_draw(int pg, int sp) { paged = pg; split = sp; }
  void connect_gr_font(Graphics_font *gf) { grf = gf; }
  void connect_graphics(Graphics *g) { gr = g; }
  int is_paged(void) { return paged; }
  int is_split(void) { return split; }
  void set_port_no(int p_no) { host_port = p_no; }
  void set_font(int fnt, int fnt_size)
   { font = fnt; font_size = fnt_size; }
  int y_off(void)
   { return (split ? (paged ? (!gr->active_page ? LO_LINES : 0) : 0) : 0); }
  int pt_in(int x, int y);
  void activate(void) { active = TRUE; }
  void inactivate(void) { active = FALSE; }
  int is_active(void) { return active; }
  void draw(int method);
  virtual void do_draw(int method = 0) = 0;
};

//----------------------------------------------------------------------
// Text prompt class
//----------------------------------------------------------------------
class Prompt : public Item
{
 private:
  int cursor_on;
  int curs_x, curs_y;
  int curs_width, curs_extra;
  int curs_hidden;
  int input_shade;
  int after_shade;
  int cursor_shade;
  int back_shade;
  int input_length;
  int first_hit;
  clock_t start_tm, end_tm;
  double elapsed_tm;
  char *str;                    // Pointer to destination string

  void pad(char *input_str, int target_length = DEFAULT_LENGTH);
  void show_input(int display_phase);

 public:
  char input_str[DEFAULT_LENGTH + 1];
  char work_str[DEFAULT_LENGTH + 1];

  Prompt(void);
  ~Prompt(void);
  void define(int x, int y, char *s, int digits);
  void set_colors(int in_clr = RED, int out_clr = BLUE,
                  int curs_clr = BLUE, int back_clr = BLACK);
  void start_prompt(void);
  void end_prompt(void);
  void position_cursor(char *after_str);
  void start_cursor(void);
  void stop_cursor(void);
  void check_cursor(void);
  int add_key(long c);
  void do_draw(int method);
};

//----------------------------------------------------------------------
// Radio button class
//----------------------------------------------------------------------
class RadioButton : public Item
{
 private:
  void do_draw(int) {}

 public:
  char *label;
  int fore_color, back_color;

  RadioButton(void); 
  ~RadioButton(void);
  void set_label(char *label, int f_color, int b_color);
};

//----------------------------------------------------------------------
// Radio button group class
//----------------------------------------------------------------------
class RadioGroup : public Item
{
 private:
  char *title;               // Group label
  int tx, ty;                // Position of group label
  int lx, ly;                // Offset of button labels
  char *on_map;              // Bitmap when a radio button is ON
  char *off_map;             // Bitmap when a radio button is OFF
  int spacing;               // Spacing between radio buttons
  int direction;             // Direction of button row HORIZ or VERT
  int left, top;             // Active boundry around each radio
  int right, bottom;         //  marking the mouse-sensitive limits
  int fore_t, back_t;        // Title colors

 public:
  RadioGroup(void);
  ~RadioGroup(void);
  RadioButton **r_button;    // RadioButton OBJECT array
  int button_tot;            // Number of buttons in set
  int was_lit;               // The previously lit button
  int lit_button;            // Currently active button

  void describe(int x, int y, int tot, int lit);
  void set_maps(char *on, char *off, int space, int dir, int l_x, int l_y);
  void set_title(char *t, int off_x, int off_y, int t_fore, int t_back);
  void set_button(int new_button)   // Change the active button
   { was_lit = lit_button; lit_button = new_button; } 
  void set_bounds(int lt, int tp, int rt, int bt)
   { left = lt; top = tp; right = rt; bottom = bt; }
  void print_label(int btn);
  void do_draw(int method);         // Draw the radio button group
};

//----------------------------------------------------------------------
// Push button base class
//----------------------------------------------------------------------
class Button : public Item
{
 public:
  Button(void);
  ~Button(void) {}
  int is_down;               // Flags that a button is currently down
  int type;                  // Button type NORMAL, SNAP_UP, or HOLD_DOWN
  int is_held_down;          // Flags that button is in the held-down state

  void describe(int sx, int sy, int width, int height, int btype);
  void activate(void);
  void inactivate(void);
  virtual void set_bmp(char *up, char *dn, char *inact = 0L) = 0;
  virtual void set_text(char *txt, int tx, int ty) = 0;
  virtual void set_colors(int text, int face, int lo_bevel, int hi_bevel) = 0;
};

//----------------------------------------------------------------------
// Push button class
//----------------------------------------------------------------------
class BMPButton : public Button
{
 private:
  char *up_bmp;              // Pointer to bitmap for button UP state
  char *dn_bmp;              // Pointer to bitmap for button DOWN state
  char *in_bmp;              // Pointer to bitmap for button INACTIVE state

 public:
  void set_bmp(char *up = 0L, char *dn = 0L, char *inact = 0L);
  void set_text(char *txt, int tx, int ty) {}
  void set_colors(int text, int face, int lo_bevel, int hi_bevel) {}
  void do_draw(int method);
};

//----------------------------------------------------------------------
// Push button class
//----------------------------------------------------------------------
class BGIButton : public Button
{
 private:
  int text_shade;            // Color of the button text
  int face_shade;            // Color of the button top face
  int lo_shade;              // Color of the dark beveled side
  int hi_shade;              // Color of the light beveled side
  char text[50];             // Button text
  int tx_pos, ty_pos;        // Relative coords of text in button face

 public:
  void set_bmp(char *up = 0L, char *dn = 0L, char *inact = 0L) {}
  void set_text(char *txt, int tx, int ty);
  void set_colors(int text, int face, int lo_bevel, int hi_bevel);
  void do_draw(int method);
};

//----------------------------------------------------------------------
// LONG Numeric display box
//----------------------------------------------------------------------
class Led : public Item
{
 private:
  long value;                // Led LONG value
  int value_is_set;          // Indicates that the LED holds a value
  char *bmp;                 // Led bitmap
  int text_shade;            // Color of the LED text
  int digits;                // Number of display digits
  int decimal;               // Number of decimal places
  long factor;               // Fixed point conversion factor
  int t_xoff, t_yoff;        // Offset of text in the LED
  int l_xoff, l_yoff;        // Offset of label outside of LED
  char *label_str;           // Led label

 public:
  Led(void);
  ~Led(void);
  void describe(char *bmap, int x, int y, int color,
                int dig, char *label = 0L);
  void set_value(long val) { value = val; value_is_set = TRUE; }
  void set_value(double val) { value = val * factor; value_is_set = TRUE; }
  void set_fixed(int dec, long fac) { decimal = dec; factor = fac; }
  void position_text(int x, int y) { t_xoff = x; t_yoff = y; }
  void position_label(int x, int y) { l_xoff = x; l_yoff = y; }
  long get_value() { return value; }
  void print(void) { draw(LED_PRINT); }
  void print(long val) { set_value(val); draw(LED_PRINT); }
  void print(double val) { set_value(val); draw(LED_PRINT); }
  void do_draw(int method);
  void activate(void);
  void inactivate(void);
};

//
// Default group record
//
struct item_group
{
  Item *start;
  Item *end;
  int type;
  int *events;
  int tot_events;
  int curr_event;
  int radio_group;
  char left_off;
  char top_off;
  char right_off;
  char bottom_off;
}; 

//----------------------------------------------------------------------
// Control Panel class
//----------------------------------------------------------------------
class Panel : public Item
{
 private:
  int tot_Buttons;           // Total number of Button controls
  int tot_Leds;              // Total number of Led controls
  int tot_Radios;            // Total number of Radio button groups

 public:
  Mouse *p_mouse;            // Pointer to active Mouse class
  Button **button;           // Array of button pointers
  Led **led;                 // Led array
  RadioGroup **radio;        // RadioGroup array
  Prompt **prompt;           // Prompts array
  Keyboard *p_keys;          // Pointer to active Keyboard class
  long key_event;            // Current event triggered by the Keyboard
  int mouse_event;           // Control event triggered by the Mouse
  int priority_event;        // Manually induced event
  int mouse_down;            // Indicates that mouse is being held down
  item_group *groups;        // Array of panel item groups
  int tot_groups;            // Total items stored in item group array
  int tot_prompts;           // Total prompt fields
  int curr_group;            // Current default item group
  int curr_prompt;           // Current active prompt field
  int def_btn_clr;           // Default color for indicating default button

  Panel(Graphics *gr, Mouse *mousePtr, Keyboard *keys, Graphics_font *gf,
        int b_cnt = 0, int l_cnt = 0, int r_cnt = 0, int p_cnt = 0,
        int paged = PAGING_OFF, int split = SPLIT_OFF);
  ~Panel(void);
  void add_BMPbutton(int sx, int sy, int width, int height, 
                     char *up_map = 0L, char *dn_map = 0L,
                     char *inact_map = 0L, int type = NORMAL_BTN);
  void add_BGIbutton(int sx, int sy, int width, int height, 
                     char *txt, int tx, int ty, int tx_clr, int face_clr, 
                     int lo_bev, int hi_bev, int font = SMALL_FONT,
                     int size = 4, int type = NORMAL_BTN);
  void add_led(char *bmap, int x, int y, int color, int digits,
               int fnt = SMALL_FONT, int fnt_size = 4,
               int btn_type = NO_BUTTONS, char *label = 0L);
  void add_radio(int x, int y, char *on, char *off,
                 int elements, int active_button,
                 char *title, int title_x, int title_y,
                 int t_fore, int t_back, int label_x, int label_y, 
                 int spacing, int direction,
                 int left, int top, int right, int bottom,
                 int fnt = SMALL_FONT, int fnt_size = 4);
  void add_prompt(int x, int y, int in_clr, int out_clr, int curs_clr,
                  char *s, int digits,
                  int fnt = SMALL_FONT, int fnt_size = 4);
  void button_up(int b_no);
  void button_down(int b_no);
  void button_inactivate(int b_no);
  void draw_default_button(int b_no, int color);
  void default_button(int b_no, int state);
  void draw_default_group(int state);
  void default_group(int state);
  int find_default_group(int event);
  void draw(void);
  void do_draw(int method) {}
  int events(void);
  long get_key(void);
  void send_event(int event) 
   { priority_event = event; }
  void clear_key(void)
   { if (p_keys) p_keys->key_code = NO_KEY; }
  virtual void define(void) = 0;
  virtual void display(void) = 0;
  virtual long key_read(void) = 0;
};
#endif  // __OP_PANEL_H

