//--------------------------------------------------
// 3D Viewer Class
//--------------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//--------------------------------------------------
#ifndef __OP_VIEW_H
#define __OP_VIEW_H
#ifndef __OP_TYPES_H
#include <op_types.h>
#endif
#ifndef __OP_VECT_H
#include <op_vect.h>
#endif

//-------------------------------------------------------------
// Viewer constants
//-------------------------------------------------------------
#define TOT_ANGLES       3600
#define ANGLE_FAC          10
#define HALF_TOT_ANGLES  (TOT_ANGLES / 2)
#define PI                3.14159265358979323846
#define CLIP_LIMIT       1000
#define PT_HIDDEN      0x8000

//-------------------------------------------------------------------------
// VIEWER
//-------------------------------------------------------------------------
class Viewer
{
 private:
  //-----------------------------------------------------------------------
  // Private data
  //-----------------------------------------------------------------------
  l_point l_eye;                // Viewer position
  d_point d_eye;                // Precise copy of Viewer position
  f_point f_eye;                // Fixed point copy of Viewer position
  l_point l_cent;               // Center of interest
  d_point d_cent;               // Precise copy of Center of interest
  f_point f_cent;               // Fixed point copy of Center of interest
  double d_x;                   // Last recorded eye/cent x-diff
  double d_y;                   // Last recorded eye/cent y-diff
  double d_z;                   // Last recorded eye/cent z-diff
  fixed f_x;                    // Last recorded eye/cent x-diff
  fixed f_y;                    // Last recorded eye/cent y-diff
  fixed f_z;                    // Last recorded eye/cent z-diff
  double distance;              // Current distance from VIEWER to CENTER
  fixed f_distance;             // Current distance from VIEWER to CENTER
  int pitch;                    // Viewer rotation about the X axis
  int bank;                     // Viewer rotation about the Z axis
  int heading;                  // Viewer rotation about the Y axis
  void clip_angles(void);       // Inhibit all three view angles
  int clip_3;                   // Flag enables use of 3-d clipping
  d_rect win;                   // Window rect of plane-normals
  long S_x, S_y;                // Window SIZE dimensions
  fixed fscreen_x, fscreen_y;   // FIXED window size dimensions
  fixed fclip_limit;            // FIXED projection limit
  fixed matrix[3][3];           // Transformation matrix
  double d_matrix[3][3];        // Floating point copy of matrix
  long l_matrix[3][3];          // Floating point copy of matrix
  double radian_fac;            // Radian to Degrees conversion factor
  double angle_fac;             // Degrees to Radian unit conversion
  double pi_2;                  // PI * 2

 public:
  //------------------------------------------------------------------------
  // Constructor / Destructor
  //------------------------------------------------------------------------
  Viewer();
  ~Viewer(void);

  //------------------------------------------------------------------------
  // Public data
  //------------------------------------------------------------------------
  int stereo_on;                // Flag enables split-view stereoscopics
  double x_travel;              // Viewer movement along the X axis
  double y_travel;              // Viewer movement along the Y axis
  double z_travel;              // Viewer movement along the Z axis
  float zoom;                   // Current viewing aperature
  int w_cx, w_cy;               // INT Window CENTER coordinates
  fixed fw_cx, fw_cy;           // FIXED Window CENTER coordinates

  //------------------------------------
  // Point of view
  //------------------------------------
  void set_position(double x, double y,      // Set position of viewpoint
                     double z);
  void set_position(d_point &pos);           // Same as above - by reference
  void get_position(d_point &pos);           // Get position - by reference
  long get_x(void) { return(l_eye.x); }      // Get current viewer X
  long get_y(void) { return(l_eye.y); }      // Get current viewer Y
  long get_z(void) { return(l_eye.z); }      // Get current viewer Z

  //------------------------------------
  // Center of interest
  //------------------------------------
  void set_center(double x, double y,        // Set center of interest
                  double z);
  double calc_distance(void);                // Calc dist from EYE to CENT
  double get_distance(void)                  // Get most recent distance
    { return distance; }                     
  double calc_radius(void);                  // Calc radius from EYE to CENT
  long get_cx(void) { return(l_cent.x); }    // Get current center X
  long get_cy(void) { return(l_cent.y); }    // Get current center Y
  long get_cz(void) { return(l_cent.z); }    // Get current center Z

  //------------------------------------
  // Viewer movement
  //------------------------------------
  void move_east(long delta);                // Increase viewer X
  void move_west(long delta);                // Decrease viewer X
  void move_north(long delta);               // Decrease viewer Z
  void move_south(long delta);               // Increase viewer Z
  void move_left(double delta);              // Move LEFT - normal
  void move_right(double delta);             // Move RIGHT - normal
  void move_forward(long delta);             // Move the eye point
  void move_up(long delta);                  // Increase viewer Y
  void move_down(long delta);                // Decrease viewer Y
  void move(l_point *st,                     // Calculate a defined move
    l_point *en, int He, int Pt, long dist); 
  void move(l_point *st,                     // Calculate a defined move
    d_point *en, int He, int Pt, double dist);
  void move(f_point *st,                     // Calculate a defined move
    f_point *en, int He, int Pt, fixed dist);
  int orbit(int delta);                      // Circular move around CENT

  //------------------------------------------------------------------------
  // Direction of view
  //------------------------------------------------------------------------
  void set_direction(int heading,            // Set viewing direction
                     int pitch, int bank);
  void view_center(void);                    // Look at center of interest
  void chg_heading(int delta);               // Change heading delta amount
  void chg_pitch(int delta);                 // Change pitch delta amount
  void chg_bank(int delta);                  // Change bank delta amount
  int get_heading(void) { return(heading); } // Get current heading
  int get_pitch(void) { return(pitch); }     // Get current pitch
  int get_bank(void) { return(bank); }       // Get current bank

  //------------------------------------------------------------------------
  // Magnification of view
  //------------------------------------------------------------------------
  void set_zoom(int zoom);                   // Set viewing aperature
  float get_zoom(void) { return(zoom); }     // Get current zoom
  float zoom_in(float delta);                // Increase view magnification
  float zoom_out(float delta);               // Decrease view maginification

  //------------------------------------------------------------------------
  // 3D calculations
  //------------------------------------------------------------------------
  void new_view(void);                       // Recalculate viewing matrix
  void cos_sin(int angle, fixed *cos,        // Retrieve sin/cos from table
               fixed *sin);
  void calc_travel(long delta);              // Calculate travel components
  void calc_travel(long delta, 
                   int pitch, int heading);  // Calculate travel components
  void clip_angle(int *angle);               // Inhibit a given angle
  int translate(l_point *v, i_screen *p,
                long *z_val);                // Translate method 1
  int translate(l_point *v, i_screen *p);    // Translate method 2
  int translate(l_point *v1, l_point *v2, 
                i_screen *p);                // Translate method 3
  int translate(f_point *v, i_screen *p,
                long *z_val);                // Translate method 4
  int translate(f_point *v, i_screen *p);    // Translate method 5
  int translate(f_point *v1, f_point *v2, 
                i_screen *p);                // Translate method 6
  void translate(s_point *v, s_point *p);    // Translate method 7
  void rotate(l_point *v, d_point *r);       // LONG to DOUBLE rotation
  void rotate(l_point *v, l_point *r);       // LONG to LONG rotation
  void rotate(d_point *v, d_point *r);       // DOUBLE to DOUBLE rotation
  void rotate(f_point *v, f_point *r);       // FIXED to FIXED rotation
  int project(d_point *v, i_screen *p);      // DOUBLE vertex projection
  int project(l_point *v, i_screen *p);      // LONG vertex projection
  int project(f_point *v, i_screen *p);      // FIXED vertex projection

  //------------------------------------------------------------------------
  // 3d Clipping
  //------------------------------------------------------------------------
  Vect *vect;                                // Private Vector geometry class
  void set_clip_zoom(float zoom);            // Set clipping pyramid
  int clip_point(d_point *pt);               // Clip POINT against window
  int clip_line(d_line ln);                  // Clip LINE against window
  int clip_poly(d_poly poly, int num_pts);   // Clip POLY against window
};
#endif  // __OP_VIEW_H
